#ifndef __CONSTANTS_H__
#define __CONSTANTS_H__
#include <vector>
#include <ncurses.h>
#include "position.h"

const int INF = 0x3F3F3F3F;

enum error {none};

// TODO: update result to include subject
// fine will waste a turn
enum result {fine, died, go_down, go_up, hit, moved,
             miss, terminate, applied, applied_nothing,
             toggle_the_world, restart_game, unknown
            };

enum game_status {terminated, main_menu, in_game, options,
                  dead, won, escaped, restart
                 };

enum game_command {game_command_terminate = 0,
                   move_north, move_south, move_east, move_west,
                   move_northeast, move_northwest,
                   move_southeast, move_southwest,
                   apply_north, apply_south, apply_east, apply_west,
                   apply_northeast, apply_northwest,
                   apply_southeast, apply_southwest,
                   apply_panic, // for curses_input specifically
                   attack_north, attack_south, attack_east, attack_west,
                   attack_northeast, attack_northwest,
                   attack_southeast, attack_southwest,
                   up_stairs, down_stairs,
                   the_world, game_restart,
                   game_command_pass, game_command_panic
                  };

enum stat_name {HP, ATK, DEF, hostile};

const int RACE_CNT = 6; // TODO: update as you go

enum race {rshade = 0, rvampire, rgoblin, rdrow, rdragon, rmerchant /* TODO: fill out the other races (including enemies) */};

// TODO: fill out the other races (including enemies)
const int  MAX_HP[RACE_CNT]        = {125, INF, 110, 150, 150, 30};
const int  STARTING_HP[RACE_CNT]   = {125, 50, 110, 150, 150, 30};
const int  STARTING_ATK[RACE_CNT]  = {25, 25, 15, 25, 20, 70};
const int  STARTING_DEF[RACE_CNT]  = {25, 25, 20, 15, 20, 5};
const char CHARACTER_REP[RACE_CNT] = {'S', 'V', 'G', 'd', 'D', 'M'};
const char *RACE_NAME[RACE_CNT] = {
        "Shade", "Vampire", "Goblin", "Drow", "Dragon"
};

const int POTION_TYPE_CNT = 6;
const int DEFAULT_POTION_TYPE_CNT = 6;
enum potion_type {restore_health = 0, boost_atk, boost_def,
                  poison_health, wound_atk, wound_def
                 };
const char *POTION_REAL_NAME[POTION_TYPE_CNT] = {
        "RH", "BA", "BD", "PH", "WA", "WD"
};

// Calculation priorities
const int CALC_ADD_BASE  = 0;
const int CALC_MUL_BASE  = 1;
const int CALC_ADD_LATER = 2;
const int CALC_MUL_LATER = 3;
const int CALC_ADD_FIXED = 4;

const int DIRECTION_CNT = 8;
// IMPORTANT: east is positive for x and SOUTH is positive for y
// initializes all directions to an int
enum direction { north = 0, south, east, west, northeast,
                 northwest, southeast, southwest
               };

const position MOVE[DIRECTION_CNT] = {
    {0, -1}, {0, 1}, {1, 0}, {-1, 0},
    {1, -1}, {-1, -1}, {1, 1}, {-1, 1}
};

const int MAP_HEIGHT     = 25;
const int MAP_WIDTH      = 79;
const int DISPLAY_HEIGHT = 30;
const int DISPLAY_WIDTH  = 79;
const int DISPLAY_BUFFER_SIZE = DISPLAY_HEIGHT * DISPLAY_WIDTH +
                                DISPLAY_WIDTH * 3;

// TODO: list all extra features
// using constants to keep track of features
// method: features: FEATURE_XXX | FEATURE_YYY
// That is: use bit manipulation
// check if feature is enabled:
// features & FEATURE_XXX
typedef unsigned int feature; // can extend to unsigned long (long) if needed
const feature FEATURE_NCURSES       = 1 << 0;
const feature FEATURE_RAND_MAP      = 1 << 1;
const feature FEATURE_ENEMIES_CHASE = 1 << 2;
const feature FEATURE_INVENTORY     = 1 << 3;
const feature FEATURE_THROW         = 1 << 4;
const feature FEATURE_REVISIT       = 1 << 5;
const feature FEATURE_LOG           = 1 << 6;
const feature FEATURE_SEED          = 1 << 7;
const feature FEATURE_MENU          = 1 << 8;
const feature FEATURE_IN_FILE       = 1 << 9;
const feature FEATURE_OUT_FILE      = 1 << 10;
const feature FEATURE_EXTRA_STUFF   = 1 << 11;
const feature FEATURE_EXTRA_LEVELS  = 1 << 12;
const feature FEATURE_COLORFUL      = 1 << 13;

const feature FEATURE_PANIC_SEED    = 1 << 27;
const feature FEATURE_PANIC_FILE    = 1 << 28;
const feature FEATURE_PANIC         = 1 << 29;
const feature FEATURE_CONFLICT      = 1 << 30;
const feature FEATURE_LIST_ARGS     = 1 << 31;

const int RETURN_FINE     = 0;
const int RETURN_PANICKED = 1;

const int COLOR_BLACK_ON_WHITE = 8;

typedef std::vector<position> position_list;
typedef std::vector<direction> direction_list;

const int GOLD_SMALL    = 1;
const int GOLD_NORMAL   = 2;
const int GOLD_MERCHANT = 4;
const int GOLD_DRAGON   = 6;

#endif
