#ifndef __GAME_H__
#define __GAME_H__
#include <memory>
#include <vector>
#include "constants.h"
#include "display.h"
#include "input.h"
#include "rng.h"
#include "characters.h"
#include "level.h"
#include "log.h"

class game final {
private:
        static const size_t MAX_LEVEL_CNT = 50;
        static const size_t MIN_LEVEL_CNT = 30;
        static const size_t DEFAULT_MAX_LEVEL = 5;
        static const size_t MAX_MSG_LENGTH = 300;

        const feature enabled_features;

        input   *in;
        display *out;
        logger  *log;
        RNG     *rng;

        unsigned int curr_turn;

        std::unique_ptr<character> player;
        std::vector<std::unique_ptr<level>> levels;

        size_t max_level;
        size_t curr_level;

        bool the_world;
        bool hostile_merchants;

        // shrink if over 300
        std::string msg;
public:
        game(const enum race starting_race,
             const feature enabled_features,
             input   *new_in,
             display *new_out,
             logger  *new_log,
             RNG     *new_rng);
        game_status run();
        size_t get_curr_level() const;
        size_t get_curr_turn() const;
        const character *get_player() const;
        void print();
private:
        result player_moves(game_command cmd);
        void new_level();
        void move_enemies();
        result player_move_or_attack(const direction &dir);
        bool compare_characters(const character *&a, const character *&b);
};

#endif
