#include "shade.h"

#include <algorithm>
#include <math.h>

shade::shade(RNG *rng, const position &pos):
        character{rng, race::rshade, pos} {
        gold = 0;
}

character::attack_result shade::attack(const direction dir,
                                       character_list &chlist) {
        position tmp{pos + MOVE[dir]};

        for (auto &ch : chlist)
                if (tmp == ch->get_position()) {
                        auto res = ch->get_hit(race, ATK, base_hit_rate);
                        return {res.res, res.dmg_dealt, res.remaining_HP, ch};
                }

        return {fine, 0, 0, nullptr};
}

character::attack_result shade::attack(const direction dir, character *ch) {
        position tmp{pos + MOVE[dir]};

        if (tmp == ch->get_position()) {
                auto res = ch->get_hit(race, ATK, base_hit_rate);
                return {res.res, res.dmg_dealt, res.remaining_HP, ch};
        }

        return {fine, 0, 0, nullptr};
}

character::hit_result shade::get_hit(const enum race &race, const int atk,
                                     const fraction hitrate) {
        if (rng->trial(hitrate)) {
                int tmp = calc_dmg(atk, DEF);

                if (tmp > HP)
                        tmp = HP;

                HP -= tmp;

                if (HP == 0)
                        return {result::died, tmp, HP};

                return {result::hit, tmp, HP};
        }

        return {result::miss, 0, HP};
}
